@@ -6,6 +6,7 @@
 
 #include "kwin_wayland_test.h"
 
+#include "backends/virtual/virtual_backend.h"
 #include "core/output.h"
 #include "core/outputbackend.h"
 #include "core/outputconfiguration.h"
@@ -132,18 +133,22 @@ private Q_SLOTS:
     void testLaptopLidClosed();
     void testGenerateConfigs_data();
     void testGenerateConfigs();
+    void testGeneratePartialConfigs();
     void testAutorotate_data();
     void testAutorotate();
     void testSettingRestoration_data();
     void testSettingRestoration();
     void testSettingRestoration_initialParsingFailure();
     void testSettingRestoration_replacedMode();
+    void testCursorRestoration();
 
     void testEvacuateTiledWindowFromRemovedOutput_data();
     void testEvacuateTiledWindowFromRemovedOutput();
 
     void testMirroring_data();
     void testMirroring();
+
+    void testAutoBrightness();
 };
 
 void OutputChangesTest::initTestCase()
@@ -152,23 +157,27 @@ void OutputChangesTest::initTestCase()
 
     QVERIFY(waylandServer()->init(s_socketName));
 
+    // delete the previous output config, to avoid previous runs messing with this one
+    // TODO reset it per test function instead?
+    QFile(QStandardPaths::locate(QStandardPaths::ConfigLocation, QStringLiteral("kwinoutputconfig.json"))).remove();
+
     kwinApp()->start();
     Test::setOutputConfig({
-        QRect(0, 0, 1280, 1024),
-        QRect(1280, 0, 1280, 1024),
+        Rect(0, 0, 1280, 1024),
+        Rect(1280, 0, 1280, 1024),
     });
 
     const auto outputs = workspace()->outputs();
     QCOMPARE(outputs.count(), 2);
-    QCOMPARE(outputs[0]->geometry(), QRect(0, 0, 1280, 1024));
-    QCOMPARE(outputs[1]->geometry(), QRect(1280, 0, 1280, 1024));
+    QCOMPARE(outputs[0]->geometry(), Rect(0, 0, 1280, 1024));
+    QCOMPARE(outputs[1]->geometry(), Rect(1280, 0, 1280, 1024));
 }
 
 void OutputChangesTest::init()
 {
     Test::setOutputConfig({
-        QRect(0, 0, 1280, 1024),
-        QRect(1280, 0, 1280, 1024),
+        Rect(0, 0, 1280, 1024),
+        Rect(1280, 0, 1280, 1024),
     });
     QVERIFY(Test::setupWaylandConnection());
 
@@ -234,7 +243,7 @@ void OutputChangesTest::testWindowSticksToOutputAfterAnotherOutputIsDisabled()
     }
     workspace()->applyOutputConfiguration(config);
 
-    QCOMPARE(workspace()->outputs().front()->geometry(), QRect(0, 0, 1280, 1024));
+    QCOMPARE(workspace()->outputs().front()->geometry(), Rect(0, 0, 1280, 1024));
 
     // The position of the window relative to its output should remain the same.
     QCOMPARE(window->frameGeometry(), RectF(42, 67, 100, 50));
@@ -591,8 +600,8 @@ void OutputChangesTest::testQuickTiledWindowRestoredAfterEnablingOutput()
     shellSurface->xdgSurface()->ack_configure(surfaceConfigureRequestedSpy.last().at(0).value<quint32>());
     Test::render(surface.get(), QSize(1280 / 2, 1024), Qt::blue);
     QVERIFY(frameGeometryChangedSpy.wait());
-    const QRectF rightQuickTileGeomScreen2 = QRectF(1280 + 1280 / 2, 0, 1280 / 2, 1024);
-    const QRectF rightQuickTileGeomScreen1 = QRectF(1280 / 2, 0, 1280 / 2, 1024);
+    const RectF rightQuickTileGeomScreen2 = RectF(1280 + 1280 / 2, 0, 1280 / 2, 1024);
+    const RectF rightQuickTileGeomScreen1 = RectF(1280 / 2, 0, 1280 / 2, 1024);
     QCOMPARE(window->frameGeometry(), rightQuickTileGeomScreen2);
     QCOMPARE(window->moveResizeGeometry(), rightQuickTileGeomScreen2);
     QCOMPARE(window->output()->backendOutput(), outputs[1]);
@@ -670,7 +679,7 @@ void OutputChangesTest::testQuickTileUntileWindowRestoredAfterEnablingOutput()
     shellSurface->xdgSurface()->ack_configure(surfaceConfigureRequestedSpy.last().at(0).value<quint32>());
     Test::render(surface.get(), QSize(100, 50), Qt::blue);
 
-    const QRectF originalGeometry = window->frameGeometry();
+    const RectF originalGeometry = window->frameGeometry();
 
     // add a second output
     {
@@ -689,7 +698,7 @@ void OutputChangesTest::testQuickTileUntileWindowRestoredAfterEnablingOutput()
     shellSurface->xdgSurface()->ack_configure(surfaceConfigureRequestedSpy.last().at(0).value<quint32>());
     Test::render(surface.get(), QSize(1280 / 2, 1024), Qt::blue);
     QVERIFY(frameGeometryChangedSpy.wait());
-    const QRectF rightQuickTileGeom = QRectF(1280 + 1280 / 2, 0, 1280 / 2, 1024);
+    const RectF rightQuickTileGeom = RectF(1280 + 1280 / 2, 0, 1280 / 2, 1024);
     QCOMPARE(window->frameGeometry(), rightQuickTileGeom);
     QCOMPARE(window->moveResizeGeometry(), rightQuickTileGeom);
     QCOMPARE(window->output()->backendOutput(), outputs[1]);
@@ -773,7 +782,7 @@ void OutputChangesTest::testCustomTiledWindowRestoredAfterEnablingOutput()
     QSignalSpy surfaceConfigureRequestedSpy(shellSurface->xdgSurface(), &Test::XdgSurface::configureRequested);
     QVERIFY(surfaceConfigureRequestedSpy.wait());
 
-    const QRectF originalGeometry = window->moveResizeGeometry();
+    const RectF originalGeometry = window->moveResizeGeometry();
 
     // Enable the right output
     {
@@ -784,7 +793,7 @@ void OutputChangesTest::testCustomTiledWindowRestoredAfterEnablingOutput()
     }
 
     QFETCH(size_t, tileIndex);
-    const QRectF customTileGeom = workspace()->rootTile(workspace()->findOutput(outputs[1]))->childTiles()[tileIndex]->windowGeometry();
+    const RectF customTileGeom = workspace()->rootTile(workspace()->findOutput(outputs[1]))->childTiles()[tileIndex]->windowGeometry();
 
     // Move the window to the right monitor and put it in the middle tile.
     QSignalSpy frameGeometryChangedSpy(window, &Window::frameGeometryChanged);
@@ -929,7 +938,7 @@ void OutputChangesTest::testMaximizeStateRestoredAfterEnablingOutput()
     QSignalSpy surfaceConfigureRequestedSpy(shellSurface->xdgSurface(), &Test::XdgSurface::configureRequested);
     QVERIFY(surfaceConfigureRequestedSpy.wait());
 
-    const QRectF originalGeometry = window->moveResizeGeometry();
+    const RectF originalGeometry = window->moveResizeGeometry();
 
     // Enable the right output
     {
@@ -1024,8 +1033,8 @@ void OutputChangesTest::testInvalidGeometryRestoreAfterEnablingOutput()
     QVERIFY(window);
     QCOMPARE(window->maximizeMode(), MaximizeFull);
 
-    const QRectF originalGeometry = window->moveResizeGeometry();
-    const QRectF originalGeometryRestore = window->geometryRestore();
+    const RectF originalGeometry = window->moveResizeGeometry();
+    const RectF originalGeometryRestore = window->geometryRestore();
 
     // Enable the right output
     {
@@ -1045,7 +1054,7 @@ void OutputChangesTest::testInvalidGeometryRestoreAfterEnablingOutput()
     QVERIFY(workspace()->findOutput(outputs[1])->geometry().contains(window->geometryRestore().topLeft().toPoint()));
     QCOMPARE(window->geometryRestore().size(), QSizeF(0, 0));
 
-    const QRectF rightGeometryRestore = window->geometryRestore();
+    const RectF rightGeometryRestore = window->geometryRestore();
 
     // Disable the right output
     {
@@ -1104,12 +1113,12 @@ void OutputChangesTest::testMaximizedWindowDoesntDisappear()
 
     Test::setOutputConfig({
         Test::OutputInfo{
-            .geometry = QRect(5120 / 3, 1440, 2256 / 1.3, 1504 / 1.3),
+            .geometry = Rect(5120 / 3, 1440, 2256 / 1.3, 1504 / 1.3),
             .scale = 1.3,
             .internal = true,
         },
         Test::OutputInfo{
-            .geometry = QRect(0, 0, 5120, 1440),
+            .geometry = Rect(0, 0, 5120, 1440),
             .scale = 1,
             .internal = false,
         },
@@ -1131,7 +1140,7 @@ void OutputChangesTest::testMaximizedWindowDoesntDisappear()
     QVERIFY(surfaceConfigureRequestedSpy.wait());
 
     window->move(workspace()->findOutput(outputs[1])->geometry().topLeft() + QPoint(3500, 500));
-    const QRectF originalGeometry = window->frameGeometry();
+    const RectF originalGeometry = window->frameGeometry();
     QVERIFY(workspace()->findOutput(outputs[1])->geometryF().contains(originalGeometry));
 
     // vertically maximize the window
@@ -1177,11 +1186,11 @@ void OutputChangesTest::testLaptopLidClosed()
 {
     Test::setOutputConfig({
         Test::OutputInfo{
-            .geometry = QRect(0, 0, 1280, 1024),
+            .geometry = Rect(0, 0, 1280, 1024),
             .internal = true,
         },
         Test::OutputInfo{
-            .geometry = QRect(1280, 0, 1280, 1024),
+            .geometry = Rect(1280, 0, 1280, 1024),
             .internal = false,
         },
     });
@@ -1213,7 +1222,7 @@ void OutputChangesTest::testLaptopLidClosed()
 }
 
 #if KWIN_BUILD_X11
-static X11Window *createX11Window(xcb_connection_t *connection, const QRect &geometry, std::function<void(xcb_window_t)> setup = {})
+static X11Window *createX11Window(xcb_connection_t *connection, const Rect &geometry, std::function<void(xcb_window_t)> setup = {})
 {
     xcb_window_t windowId = xcb_generate_id(connection);
     xcb_create_window(connection, XCB_COPY_FROM_PARENT, windowId, rootWindow(),
@@ -1247,8 +1256,8 @@ void OutputChangesTest::testXwaylandScaleChange()
 {
 #if KWIN_BUILD_X11
     Test::setOutputConfig({
-        QRect(0, 0, 1280, 1024),
-        QRect(1280, 0, 1280, 1024),
+        Rect(0, 0, 1280, 1024),
+        Rect(1280, 0, 1280, 1024),
     });
     const auto outputs = kwinApp()->outputBackend()->outputs();
 
@@ -1262,8 +1271,8 @@ void OutputChangesTest::testXwaylandScaleChange()
 
     Test::XcbConnectionPtr c = Test::createX11Connection();
     QVERIFY(!xcb_connection_has_error(c.get()));
-    X11Window *window = createX11Window(c.get(), QRect(0, 0, 100, 200));
-    const QRectF originalGeometry = window->frameGeometry();
+    X11Window *window = createX11Window(c.get(), Rect(0, 0, 100, 200));
+    const RectF originalGeometry = window->frameGeometry();
 
     // disable the left output -> window gets moved to the right output
     {
@@ -1309,7 +1318,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("1080p 27\"")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 1920, 1080),
+               .geometry = Rect(0, 0, 1920, 1080),
                .internal = false,
                .physicalSizeInMM = QSize(598, 336),
                .modes = {ModeInfo(QSize(1920, 1080), 60000, OutputMode::Flag::Preferred)},
@@ -1319,7 +1328,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("2160p 27\"")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 3840, 2160),
+               .geometry = Rect(0, 0, 3840, 2160),
                .internal = false,
                .physicalSizeInMM = QSize(598, 336),
                .modes = {ModeInfo(QSize(3840, 2160), 60000, OutputMode::Flag::Preferred)},
@@ -1329,7 +1338,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("2160p invalid size")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 3840, 2160),
+               .geometry = Rect(0, 0, 3840, 2160),
                .internal = false,
                .physicalSizeInMM = QSize(),
                .modes = {ModeInfo(QSize(3840, 2160), 60000, OutputMode::Flag::Preferred)},
@@ -1339,7 +1348,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("2160p impossibly tiny size")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 3840, 2160),
+               .geometry = Rect(0, 0, 3840, 2160),
                .internal = false,
                .physicalSizeInMM = QSize(1, 1),
                .modes = {ModeInfo(QSize(3840, 2160), 60000, OutputMode::Flag::Preferred)},
@@ -1349,7 +1358,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("1080p 27\" with non-preferred high refresh option")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 1920, 1080),
+               .geometry = Rect(0, 0, 1920, 1080),
                .internal = false,
                .physicalSizeInMM = QSize(598, 336),
                .modes = {ModeInfo(QSize(1920, 1080), 60000, OutputMode::Flag::Preferred), ModeInfo(QSize(1920, 1080), 120000, OutputMode::Flags{})},
@@ -1359,7 +1368,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("2160p 27\" with 30Hz preferred mode")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 3840, 2160),
+               .geometry = Rect(0, 0, 3840, 2160),
                .internal = false,
                .physicalSizeInMM = QSize(598, 336),
                .modes = {ModeInfo(QSize(3840, 2160), 30000, OutputMode::Flag::Preferred), ModeInfo(QSize(2560, 1440), 60000, OutputMode::Flags{})},
@@ -1369,7 +1378,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("2160p 27\" with 30Hz preferred and a generated 60Hz mode")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 3840, 2160),
+               .geometry = Rect(0, 0, 3840, 2160),
                .internal = false,
                .physicalSizeInMM = QSize(598, 336),
                .modes = {ModeInfo(QSize(3840, 2160), 30000, OutputMode::Flag::Preferred), ModeInfo(QSize(2560, 1440), 60000, OutputMode::Flag::Generated)},
@@ -1379,7 +1388,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("1440p 32:9 49\" with two preferred modes")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 5120, 1440),
+               .geometry = Rect(0, 0, 5120, 1440),
                .internal = false,
                .physicalSizeInMM = QSize(1190, 340),
                .modes = {ModeInfo(QSize(3840, 1080), 120000, OutputMode::Flag::Preferred), ModeInfo(QSize(5120, 1440), 120000, OutputMode::Flag::Preferred)},
@@ -1389,7 +1398,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("2160p 32:9 57\" with non-native preferred mode")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 7680, 2160),
+               .geometry = Rect(0, 0, 7680, 2160),
                .internal = false,
                .physicalSizeInMM = QSize(1400, 400),
                .modes = {ModeInfo(QSize(3840, 1080), 60000, OutputMode::Flag::Preferred), ModeInfo(QSize(7680, 2160), 120000, OutputMode::Flags{})},
@@ -1399,7 +1408,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("Framework 1920p 13.5\"")
         << DeviceType::Laptop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 2880, 1920),
+               .geometry = Rect(0, 0, 2880, 1920),
                .internal = true,
                .physicalSizeInMM = QSize(285, 190),
                .modes = {ModeInfo(QSize(2880, 1920), 120000, OutputMode::Flag::Preferred)},
@@ -1409,7 +1418,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("DELL XPS 13 1080p 13\"")
         << DeviceType::Laptop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 1920, 1080),
+               .geometry = Rect(0, 0, 1920, 1080),
                .internal = true,
                .physicalSizeInMM = QSize(293, 162),
                .modes = {ModeInfo(QSize(1920, 1080), 60000, OutputMode::Flag::Preferred)},
@@ -1419,7 +1428,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("DELL XPS 13 2160p 13\"")
         << DeviceType::Laptop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 3840, 2160),
+               .geometry = Rect(0, 0, 3840, 2160),
                .internal = true,
                .physicalSizeInMM = QSize(294, 165),
                .modes = {ModeInfo(QSize(3840, 2160), 60000, OutputMode::Flag::Preferred)},
@@ -1429,7 +1438,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("ThinkPad T14 2400p 14\"")
         << DeviceType::Laptop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 3840, 2400),
+               .geometry = Rect(0, 0, 3840, 2400),
                .internal = true,
                .physicalSizeInMM = QSize(301, 188),
                .modes = {ModeInfo(QSize(3840, 2400), 60000, OutputMode::Flag::Preferred)},
@@ -1439,7 +1448,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("SteamDeck OLED")
         << DeviceType::Laptop
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 800, 1280),
+               .geometry = Rect(0, 0, 800, 1280),
                .internal = true,
                .physicalSizeInMM = QSize(100, 160),
                .modes = {ModeInfo(QSize(800, 1280), 90000, OutputMode::Flag::Preferred)},
@@ -1450,7 +1459,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("Pixel 3a")
         << DeviceType::Phone
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 1080, 2220),
+               .geometry = Rect(0, 0, 1080, 2220),
                .internal = true,
                .physicalSizeInMM = QSize(62, 128),
                .modes = {ModeInfo(QSize(1080, 2220), 60000, OutputMode::Flags{}), ModeInfo(QSize(1080, 2220), 120000, OutputMode::Flag::Preferred)},
@@ -1460,7 +1469,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("OnePlus 6")
         << DeviceType::Phone
         << Test::OutputInfo{
-               .geometry = QRect(0, 0, 1080, 2280),
+               .geometry = Rect(0, 0, 1080, 2280),
                .internal = true,
                .physicalSizeInMM = QSize(68, 145),
                .modes = {ModeInfo(QSize(1080, 2280), 60000, OutputMode::Flag::Preferred)},
@@ -1470,7 +1479,7 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("Samsung Odyssey G5")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(),
+               .geometry = Rect(),
                .internal = false,
                .physicalSizeInMM = QSize(698, 393),
                .modes = {ModeInfo(QSize(2560, 1440), 164831, OutputMode::Flag::Preferred)},
@@ -1481,12 +1490,36 @@ void OutputChangesTest::testGenerateConfigs_data()
     QTest::addRow("LG C4 77\"")
         << DeviceType::Desktop
         << Test::OutputInfo{
-               .geometry = QRect(),
+               .geometry = Rect(),
                .internal = false,
                .physicalSizeInMM = QSize(1600, 900),
                .modes = {ModeInfo(QSize(3840, 2160), 120000, OutputMode::Flag::Preferred)},
            }
         << ModeInfo(QSize(3840, 2160), 120000, OutputMode::Flag::Preferred) << 2.0 << true;
+
+    QTest::addRow("Acer 24 CB242Ybmiprx")
+        << DeviceType::Desktop
+        << Test::OutputInfo{
+               .geometry = QRect(),
+               .internal = false,
+               .physicalSizeInMM = QSize(527, 296),
+               .modes = {
+                   ModeInfo(QSize(1920, 1080), 60000, OutputMode::Flag::Preferred),
+                   ModeInfo(QSize(1920, 1080), 75000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1920, 1080), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1920, 1080), 50000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1680, 1050), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1280, 1024), 75000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1280, 1024), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1440, 900), 50000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1280, 960), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1920, 540), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1280, 800), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1152, 864), 75000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1280, 720), 60000, OutputMode::Flags{}),
+               },
+           }
+        << ModeInfo(QSize(1920, 1080), 75000, OutputMode::Flags{}) << 1.0 << true;
 }
 
 void OutputChangesTest::testGenerateConfigs()
@@ -1534,6 +1567,77 @@ void OutputChangesTest::testGenerateConfigs()
     QCOMPARE(*outputConfig->allowDdcCi, defaultDDCValue);
 }
 
+void OutputChangesTest::testGeneratePartialConfigs()
+{
+    // This test verifies that adding an output to an existing configuration
+    // keeps some properties of that configuration (position, priority)
+    // instead of generating a completely new one
+
+    // TODO change test API so it's possible to add outputs without also configuring them
+    const auto outputBackend = qobject_cast<VirtualBackend *>(kwinApp()->outputBackend());
+    outputBackend->setVirtualOutputs({
+        VirtualBackend::OutputInfo{
+            .geometry = Rect(0, 0, 1920, 1080),
+            .edid = readEdid(QFINDTESTDATA("data/Odyssey G5.bin")),
+            .edidIdentifierOverride = QByteArrayLiteral("GeneratePartialConfigs-1"),
+        },
+        VirtualBackend::OutputInfo{
+            .geometry = Rect(0, 0, 1920, 1080),
+            .edid = readEdid(QFINDTESTDATA("data/Odyssey G5.bin")),
+            .edidIdentifierOverride = QByteArrayLiteral("GeneratePartialConfigs-2"),
+        },
+    });
+    auto outputs = kwinApp()->outputBackend()->outputs();
+
+    // workspace should have the outputs configured to be next to each other,
+    // with default priority in the order of the outputs
+    QCOMPARE(outputs[0]->position(), QPoint(0, 0));
+    QCOMPARE(outputs[0]->priority(), 0);
+    QCOMPARE(outputs[1]->position(), QPoint(1920, 0));
+    QCOMPARE(outputs[1]->priority(), 1);
+
+    {
+        // change the priority values and positions
+        OutputConfiguration config;
+        *config.changeSet(outputs[0]) = OutputChangeSet{
+            .pos = QPoint(0, 1080),
+            .priority = 1,
+        };
+        *config.changeSet(outputs[1]) = OutputChangeSet{
+            .pos = QPoint(500, 0),
+            .priority = 0,
+        };
+        QCOMPARE(workspace()->applyOutputConfiguration(config), OutputConfigurationError::None);
+    }
+
+    // now add another output
+    outputBackend->setVirtualOutputs({
+        VirtualBackend::OutputInfo{
+            .geometry = Rect(0, 0, 1920, 1080),
+            .edid = readEdid(QFINDTESTDATA("data/Odyssey G5.bin")),
+            .edidIdentifierOverride = QByteArrayLiteral("GeneratePartialConfigs-1"),
+        },
+        VirtualBackend::OutputInfo{
+            .geometry = Rect(0, 0, 1920, 1080),
+            .edid = readEdid(QFINDTESTDATA("data/Odyssey G5.bin")),
+            .edidIdentifierOverride = QByteArrayLiteral("GeneratePartialConfigs-2"),
+        },
+        VirtualBackend::OutputInfo{
+            .geometry = Rect(0, 0, 1920, 1080),
+        },
+    });
+    outputs = kwinApp()->outputBackend()->outputs();
+
+    // position and priority should still be what we applied before
+    QCOMPARE(outputs[0]->position(), QPoint(0, 1080));
+    QCOMPARE(outputs[0]->priority(), 1);
+    QCOMPARE(outputs[1]->position(), QPoint(500, 0));
+    QCOMPARE(outputs[1]->priority(), 0);
+    // the new output should also have sane default values
+    QCOMPARE(outputs[2]->position(), QPoint(2420, 0));
+    QCOMPARE(outputs[2]->priority(), 2);
+}
+
 void OutputChangesTest::testAutorotate_data()
 {
     QTest::addColumn<OutputTransform::Kind>("panelOrientation");
@@ -1568,7 +1672,7 @@ void OutputChangesTest::testAutorotate()
 
     QFETCH(OutputTransform::Kind, panelOrientation);
     Test::setOutputConfig({Test::OutputInfo{
-        .geometry = QRect(0, 0, 1280, 1024),
+        .geometry = Rect(0, 0, 1280, 1024),
         .internal = true,
         .physicalSizeInMM = QSize(598, 336),
         .modes = {ModeInfo(QSize(1280, 1024), 60000, OutputMode::Flag::Preferred)},
@@ -1762,7 +1866,7 @@ void OutputChangesTest::testSettingRestoration()
 
     Test::setOutputConfig(outputData | std::views::transform([](const IdentificationData &data) {
         return Test::OutputInfo{
-            .geometry = QRect(0, 0, 1280, 1024),
+            .geometry = Rect(0, 0, 1280, 1024),
             .internal = false,
             .physicalSizeInMM = QSize(598, 336),
             .modes = {ModeInfo(QSize(1280, 1024), 60000, OutputMode::Flag::Preferred)},
@@ -1805,7 +1909,7 @@ void OutputChangesTest::testSettingRestoration()
     // this must work if one of the outputs is removed in between as well
     Test::setOutputConfig({
         Test::OutputInfo{
-            .geometry = QRect(1280, 0, 1280, 1024),
+            .geometry = Rect(1280, 0, 1280, 1024),
             .internal = false,
             .physicalSizeInMM = QSize(598, 336),
             .modes = {ModeInfo(QSize(1280, 1024), 60000, OutputMode::Flag::Preferred)},
@@ -1825,7 +1929,7 @@ void OutputChangesTest::testSettingRestoration()
     // and add it again, with the inverted order
     Test::setOutputConfig(outputData | std::views::reverse | std::views::transform([](const IdentificationData &data) {
         return Test::OutputInfo{
-            .geometry = QRect(0, 0, 1280, 1024),
+            .geometry = Rect(0, 0, 1280, 1024),
             .internal = false,
             .physicalSizeInMM = QSize(598, 336),
             .modes = {ModeInfo(QSize(1280, 1024), 60000, OutputMode::Flag::Preferred)},
@@ -1862,7 +1966,7 @@ void OutputChangesTest::testSettingRestoration_initialParsingFailure()
     // to additionally test the case when EDID ID isn't unique when this happens
     Test::setOutputConfig({
         Test::OutputInfo{
-            .geometry = QRect(0, 0, 1280, 1024),
+            .geometry = Rect(0, 0, 1280, 1024),
             .internal = false,
             .physicalSizeInMM = QSize(598, 336),
             .modes = {
@@ -1875,7 +1979,7 @@ void OutputChangesTest::testSettingRestoration_initialParsingFailure()
             .mstPath = QByteArrayLiteral("MST-1-1"),
         },
         Test::OutputInfo{
-            .geometry = QRect(0, 0, 1280, 1024),
+            .geometry = Rect(0, 0, 1280, 1024),
             .internal = false,
             .physicalSizeInMM = QSize(598, 336),
             .modes = {
@@ -1927,7 +2031,7 @@ void OutputChangesTest::testSettingRestoration_initialParsingFailure()
     // now libdisplay-info was updated, and we have an EDID ID for the same hash
     Test::setOutputConfig({
         Test::OutputInfo{
-            .geometry = QRect(0, 0, 1280, 1024),
+            .geometry = Rect(0, 0, 1280, 1024),
             .internal = false,
             .physicalSizeInMM = QSize(598, 336),
             .modes = {
@@ -1940,7 +2044,7 @@ void OutputChangesTest::testSettingRestoration_initialParsingFailure()
             .mstPath = QByteArrayLiteral("MST-1-1"),
         },
         Test::OutputInfo{
-            .geometry = QRect(0, 0, 1280, 1024),
+            .geometry = Rect(0, 0, 1280, 1024),
             .internal = false,
             .physicalSizeInMM = QSize(598, 336),
             .modes = {
@@ -1967,7 +2071,7 @@ void OutputChangesTest::testSettingRestoration_replacedMode()
 {
     Test::setOutputConfig({
         Test::OutputInfo{
-            .geometry = QRect(0, 0, 1280, 1024),
+            .geometry = Rect(0, 0, 1280, 1024),
             .internal = false,
             .physicalSizeInMM = QSize(598, 336),
             .modes = {
@@ -2015,6 +2119,43 @@ void OutputChangesTest::testSettingRestoration_replacedMode()
     QCOMPARE(output->currentMode(), output->modes()[2]);
 }
 
+void OutputChangesTest::testCursorRestoration()
+{
+    // This test verifies that the cursor gets put back to its original position
+    // when output changes happen, even with edge barriers enabled
+    options->setEdgeBarrier(100);
+
+    Test::setOutputConfig({
+        Test::OutputInfo{
+            .geometry = Rect(0, 0, 2880, 1920),
+            .scale = 1.6,
+            .connectorName = QStringLiteral("eDP-1"),
+        },
+    });
+
+    BackendOutput *tmp = kwinApp()->outputBackend()->createVirtualOutput("DP-1", "", QSize(5120, 1440), 1.0);
+
+    auto outputs = kwinApp()->outputBackend()->outputs();
+
+    OutputConfiguration config;
+    config.changeSet(outputs[0])->pos = QPoint(1691, 1440);
+    config.changeSet(outputs[1])->pos = QPoint(0, 0);
+    QCOMPARE(workspace()->applyOutputConfiguration(config), OutputConfigurationError::None);
+
+    input()->pointer()->warp(outputs[0]->position() + QPoint(1500, 1000));
+
+    // if an unrelated output is removed, the cursor should stay where it was
+    // relative to the output it's on
+    kwinApp()->outputBackend()->removeVirtualOutput(tmp);
+    outputs = kwinApp()->outputBackend()->outputs();
+    QCOMPARE(input()->pointer()->pos(), outputs[0]->position() + QPoint(1500, 1000));
+
+    // same when it's added back
+    tmp = kwinApp()->outputBackend()->createVirtualOutput("DP-1", "", QSize(5120, 1440), 1.0);
+    outputs = kwinApp()->outputBackend()->outputs();
+    QCOMPARE(input()->pointer()->pos(), outputs[0]->position() + QPoint(1500, 1000));
+}
+
 void OutputChangesTest::testEvacuateTiledWindowFromRemovedOutput_data()
 {
     QTest::addColumn<QuickTileFlag>("tileMode");
@@ -2032,11 +2173,11 @@ void OutputChangesTest::testEvacuateTiledWindowFromRemovedOutput()
 {
     Test::setOutputConfig({
         Test::OutputInfo{
-            .geometry = QRect(0, 0, 5120, 1440),
+            .geometry = Rect(0, 0, 5120, 1440),
             .internal = false,
         },
         Test::OutputInfo{
-            .geometry = QRect(1705, 1440, 1800, 1200),
+            .geometry = Rect(1705, 1440, 1800, 1200),
             .scale = 1.6,
             .internal = true,
         },
@@ -2079,7 +2220,7 @@ void OutputChangesTest::testEvacuateTiledWindowFromRemovedOutput()
         QVERIFY(workspace()->findOutput(external)->geometryF().contains(window->frameGeometry()));
     }
 
-    const QRectF originalGeometry = window->frameGeometry();
+    const RectF originalGeometry = window->frameGeometry();
 
     // now remove the external output
     {
@@ -2141,12 +2282,12 @@ void OutputChangesTest::testMirroring()
 
     Test::setOutputConfig({
         Test::OutputInfo{
-            .geometry = QRect(0, 0, 1280, 1200),
+            .geometry = Rect(0, 0, 1280, 1200),
             .scale = 1.0,
             .internal = true,
         },
         Test::OutputInfo{
-            .geometry = QRect(QPoint(1280, 0), resolution),
+            .geometry = Rect(QPoint(1280, 0), resolution),
             .scale = 1.0,
             .internal = false,
         },
@@ -2188,6 +2329,48 @@ void OutputChangesTest::testMirroring()
     input()->removeInputDevice(&lidSwitch);
 }
 
+#define COMPARE_RANGE(expression, value, uncertainty) \
+    QCOMPARE_GE(expression, value - uncertainty);     \
+    QCOMPARE_LE(expression, value + uncertainty);
+
+void OutputChangesTest::testAutoBrightness()
+{
+    AutoBrightnessCurve curve;
+    curve.adjust(1.00, 100);
+    curve.adjust(0.75, 50);
+    curve.adjust(0.40, 10);
+    curve.adjust(0.20, 1);
+
+    constexpr double eta = 0.001;
+
+    COMPARE_RANGE(curve.sample(100), 1.00, eta);
+    COMPARE_RANGE(curve.sample(50), 0.75, eta);
+    COMPARE_RANGE(curve.sample(10), 0.40, eta);
+    COMPARE_RANGE(curve.sample(1), 0.20, eta);
+
+    // reduce brightness at higher lux values
+    curve.adjust(0.40, 0);
+    curve.adjust(0.20, 10);
+    COMPARE_RANGE(curve.sample(10), 0.20, eta);
+
+    // reduce brightness at zero lux
+    curve.adjust(0.40, 0);
+    curve.adjust(0.35, 0);
+    curve.adjust(0.20, 0);
+    COMPARE_RANGE(curve.sample(0), 0.20, eta);
+
+    // increase brightness at zero lux
+    curve.adjust(0.20, 0);
+    curve.adjust(0.25, 0);
+    curve.adjust(0.30, 0);
+    curve.adjust(0.35, 0);
+    COMPARE_RANGE(curve.sample(0), 0.35, eta);
+
+    // higher luminance values should be unaffected by the changes at low brightness
+    COMPARE_RANGE(curve.sample(100), 1.00, eta);
+    COMPARE_RANGE(curve.sample(50), 0.75, eta);
+}
+
 } // namespace KWin
 
 WAYLANDTEST_MAIN(KWin::OutputChangesTest)
