@@ -165,7 +165,7 @@ Region X11WindowedOutput::exposedArea() const
     return m_exposedArea;
 }
 
-void X11WindowedOutput::addExposedArea(const QRect &rect)
+void X11WindowedOutput::addExposedArea(const Rect &rect)
 {
     m_exposedArea += rect;
 }
@@ -490,6 +490,8 @@ bool X11WindowedOutput::present(const QList<OutputLayer *> &layersToUpdate, cons
                        0,
                        0,
                        nullptr);
+
+    Q_ASSERT(!m_frame);
     m_frame = frame;
     return true;
 }
