@@ -546,6 +546,7 @@ void DebugConsoleFilter::tabletToolProximityEvent(TabletToolProximityEvent *even
         + tableRow(i18n("Tilt"),
                    QStringLiteral("%1,%2").arg(event->xTilt).arg(event->yTilt))
         + tableRow(i18n("Rotation"), QString::number(event->rotation))
+        + tableRow(i18n("Distance"), QString::number(event->distance))
         + s_tableEnd;
 
     m_textEdit->insertHtml(text);
@@ -564,7 +565,7 @@ void DebugConsoleFilter::tabletToolAxisEvent(TabletToolAxisEvent *event)
                    QStringLiteral("%1,%2").arg(event->xTilt).arg(event->yTilt))
         + tableRow(i18n("Rotation"), QString::number(event->rotation))
         + tableRow(i18n("Pressure"), QString::number(event->pressure))
-        + tableRow(i18n("Buttons"), QString::number(event->buttons))
+        + tableRow(i18n("Distance"), QString::number(event->distance))
         + s_tableEnd;
 
     m_textEdit->insertHtml(text);
@@ -584,9 +585,7 @@ void DebugConsoleFilter::tabletToolTipEvent(TabletToolTipEvent *event)
                    QStringLiteral("%1,%2").arg(event->xTilt).arg(event->yTilt))
         + tableRow(i18n("Rotation"), QString::number(event->rotation))
         + tableRow(i18n("Pressure"), QString::number(event->pressure))
-        + tableRow(i18n("Distance"), QString::number(event->distance))
         + tableRow(i18n("Slider Position"), QString::number(event->sliderPosition))
-        + tableRow(i18n("Buttons"), QString::number(event->buttons))
         + s_tableEnd;
 
     m_textEdit->insertHtml(text);
@@ -889,7 +888,7 @@ QString DebugConsoleDelegate::displayText(const QVariant &value, const QLocale &
     }
     default:
         if (value.userType() == qMetaTypeId<QStringList>()) {
-            return value.toStringList().join(QLatin1String(", "));
+            return value.toStringList().join(QLatin1StringView(", "));
         }
         if (value.userType() == qMetaTypeId<KWin::SurfaceInterface *>()) {
             if (auto s = value.value<KWin::SurfaceInterface *>()) {
@@ -907,14 +906,14 @@ QString DebugConsoleDelegate::displayText(const QVariant &value, const QLocale &
         }
         if (value.userType() == qMetaTypeId<KWin::LogicalOutput *>()) {
             if (auto output = value.value<KWin::LogicalOutput *>()) {
-                return QStringLiteral("%1 (%2@%3x)").arg(output->name(), displayText(output->geometry(), locale), QString::number(output->scale()));
+                return QStringLiteral("%1 (%2@%3x)").arg(output->name(), displayText(QVariant::fromValue(output->geometry()), locale), QString::number(output->scale()));
             } else {
                 return QStringLiteral("nullptr");
             }
         }
         if (value.userType() == qMetaTypeId<KWin::Tile *>()) {
             if (auto tile = value.value<KWin::Tile *>()) {
-                const QString tileGeometry = displayText(tile->absoluteGeometry(), locale);
+                const QString tileGeometry = displayText(QVariant::fromValue(tile->absoluteGeometry()), locale);
 
                 if (auto customTile = qobject_cast<KWin::CustomTile *>(tile)) {
                     const QString direction = QMetaEnum::fromType<KWin::Tile::LayoutDirection>().valueToKey(static_cast<quint64>(customTile->layoutDirection()));
@@ -1042,6 +1041,14 @@ QString DebugConsoleDelegate::displayText(const QVariant &value, const QLocale &
             }
             return list.join(QStringLiteral(", "));
         }
+        if (value.userType() == qMetaTypeId<Rect>()) {
+            const Rect r = value.value<Rect>();
+            return QStringLiteral("%1,%2 %3x%4").arg(r.x()).arg(r.y()).arg(r.width()).arg(r.height());
+        }
+        if (value.userType() == qMetaTypeId<RectF>()) {
+            const RectF r = value.value<RectF>();
+            return QStringLiteral("%1,%2 %3x%4").arg(r.x()).arg(r.y()).arg(r.width()).arg(r.height());
+        }
         break;
     }
     return QStyledItemDelegate::displayText(value, locale);
@@ -1678,7 +1685,7 @@ QVariant DataSourceModel::data(const QModelIndex &index, int role) const
         return mimeType;
     } else if (index.column() == 1 && index.row() < m_data.count()) {
         const QByteArray &data = m_data.at(index.row());
-        if (mimeType.contains(QLatin1String("image"))) {
+        if (mimeType.contains(QLatin1StringView("image"))) {
             if (role == Qt::DecorationRole) {
                 return QImage::fromData(data);
             }
