@@ -422,7 +422,7 @@ void AnimationEffect::genericAnimation(EffectWindow *w, WindowPaintData &data, f
 {
 }
 
-static qreal xCoord(const QRectF &r, int flag)
+static qreal xCoord(const RectF &r, int flag)
 {
     if (flag & AnimationEffect::Left) {
         return r.x();
@@ -433,7 +433,7 @@ static qreal xCoord(const QRectF &r, int flag)
     }
 }
 
-static qreal yCoord(const QRectF &r, int flag)
+static qreal yCoord(const RectF &r, int flag)
 {
     if (flag & AnimationEffect::Top) {
         return r.y();
@@ -444,9 +444,9 @@ static qreal yCoord(const QRectF &r, int flag)
     }
 }
 
-QRect AnimationEffect::clipRect(const QRect &geo, const AniData &anim) const
+Rect AnimationEffect::clipRect(const Rect &geo, const AniData &anim) const
 {
-    QRect clip = geo;
+    Rect clip = geo;
     FPx2 ratio = anim.from + progress(anim) * (anim.to - anim.from);
     if (anim.from[0] < 1.0 || anim.to[0] < 1.0) {
         clip.setWidth(clip.width() * ratio[0]);
@@ -454,8 +454,8 @@ QRect AnimationEffect::clipRect(const QRect &geo, const AniData &anim) const
     if (anim.from[1] < 1.0 || anim.to[1] < 1.0) {
         clip.setHeight(clip.height() * ratio[1]);
     }
-    const QRect center = geo.adjusted(clip.width() / 2, clip.height() / 2,
-                                      -(clip.width() + 1) / 2, -(clip.height() + 1) / 2);
+    const Rect center = geo.adjusted(clip.width() / 2, clip.height() / 2,
+                                     -(clip.width() + 1) / 2, -(clip.height() + 1) / 2);
     const qreal x[2] = {xCoord(center, metaData(SourceAnchor, anim.meta)),
                         xCoord(center, metaData(TargetAnchor, anim.meta))};
     const qreal y[2] = {yCoord(center, metaData(SourceAnchor, anim.meta)),
@@ -501,14 +501,14 @@ static inline float geometryCompensation(int flags, float v)
     return 0.5 * (1.0 - v); // half compensation
 }
 
-void AnimationEffect::paintWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, const QRegion &deviceRegion, WindowPaintData &data)
+void AnimationEffect::paintWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, const Region &deviceRegion, WindowPaintData &data)
 {
     auto it = d->m_animations.find(w);
     if (it == d->m_animations.end()) {
         effects->paintWindow(renderTarget, viewport, w, mask, deviceRegion, data);
         return;
     }
-    QRegion effectiveDeviceRegion = deviceRegion;
+    Region effectiveDeviceRegion = deviceRegion;
     auto &[window, pair] = *it;
     auto &[list, rect] = pair;
     for (auto &anim : list) {
@@ -974,7 +974,7 @@ QString AnimationEffect::debug(const QString &parameter) const
             if (caption.isEmpty()) {
                 caption = QStringLiteral("[Untitled]");
             }
-            dbg += QLatin1String("Animating window: ") + caption + QLatin1Char('\n');
+            dbg += QLatin1StringView("Animating window: ") + caption + QLatin1Char('\n');
             for (const auto &anim : data) {
                 dbg += anim.debugInfo();
             }
