@@ -158,11 +158,11 @@ void OutputConfigurationStore::applyMirroring(OutputConfiguration &config, const
     }
 }
 
-std::optional<std::pair<OutputConfigurationStore::Setup *, std::unordered_map<BackendOutput *, size_t>>> OutputConfigurationStore::findSetup(const QList<BackendOutput *> &outputs, bool lidClosed)
+std::optional<OutputConfigurationStore::SetupWithOutputs> OutputConfigurationStore::findSetup(const QList<BackendOutput *> &outputs, bool lidClosed)
 {
     std::unordered_map<BackendOutput *, size_t> outputStates;
     for (BackendOutput *output : outputs) {
-        if (auto opt = findOutput(output, outputs)) {
+        if (auto opt = findOutputIndex(output, outputs)) {
             outputStates[output] = *opt;
         } else {
             return std::nullopt;
@@ -181,11 +181,14 @@ std::optional<std::pair<OutputConfigurationStore::Setup *, std::unordered_map<Ba
     if (setup == m_setups.end()) {
         return std::nullopt;
     } else {
-        return std::make_pair(&(*setup), outputStates);
+        return SetupWithOutputs{
+            .setup = &*setup,
+            .globalOutputIndices = outputStates,
+        };
     }
 }
 
-std::optional<size_t> OutputConfigurationStore::findOutput(BackendOutput *output, const QList<BackendOutput *> &allOutputs) const
+std::optional<size_t> OutputConfigurationStore::findOutputIndex(BackendOutput *output, const QList<BackendOutput *> &allOutputs) const
 {
     struct Properties
     {
@@ -298,14 +301,14 @@ void OutputConfigurationStore::storeConfig(const QList<BackendOutput *> &allOutp
     const auto opt = findSetup(relevantOutputs, isLidClosed);
     Setup *setup = nullptr;
     if (opt) {
-        setup = opt->first;
+        setup = opt->setup;
     } else {
         m_setups.push_back(Setup{});
         setup = &m_setups.back();
         setup->lidClosed = isLidClosed;
     }
     for (BackendOutput *output : relevantOutputs) {
-        auto outputIndex = findOutput(output, allOutputs);
+        auto outputIndex = findOutputIndex(output, allOutputs);
         Q_ASSERT(outputIndex.has_value());
         auto outputIt = std::find_if(setup->outputs.begin(), setup->outputs.end(), [outputIndex](const auto &output) {
             return output.outputIndex == outputIndex;
@@ -358,6 +361,8 @@ void OutputConfigurationStore::storeConfig(const QList<BackendOutput *> &allOutp
                 .maxBitsPerColor = changeSet->maxBitsPerColor.value_or(output->maxBitsPerColor()),
                 .edrPolicy = changeSet->edrPolicy.value_or(output->edrPolicy()),
                 .sharpness = changeSet->sharpness.value_or(output->sharpnessSetting()),
+                .automaticBrightness = changeSet->automaticBrightness.value_or(output->automaticBrightness()),
+                .autoBrightnessCurve = changeSet->autoBrightnessCurve.value_or(output->autoBrightnessCurve()),
             };
             *outputIt = SetupState{
                 .outputIndex = *outputIndex,
@@ -409,6 +414,8 @@ void OutputConfigurationStore::storeConfig(const QList<BackendOutput *> &allOutp
                 .maxBitsPerColor = output->maxBitsPerColor(),
                 .edrPolicy = output->edrPolicy(),
                 .sharpness = output->sharpnessSetting(),
+                .automaticBrightness = output->automaticBrightness(),
+                .autoBrightnessCurve = output->autoBrightnessCurve(),
             };
             *outputIt = SetupState{
                 .outputIndex = *outputIndex,
@@ -479,6 +486,8 @@ OutputConfiguration OutputConfigurationStore::setupToConfig(Setup *setup, const
             .edrPolicy = state.edrPolicy,
             .sharpness = state.sharpness,
             .priority = setupState.priority,
+            .automaticBrightness = state.automaticBrightness,
+            .autoBrightnessCurve = state.autoBrightnessCurve,
         };
     }
     return ret;
@@ -497,7 +506,7 @@ std::optional<OutputConfiguration> OutputConfigurationStore::generateLidClosedCo
         return std::nullopt;
     }
     BackendOutput *const internalOutput = *internalIt;
-    auto config = setupToConfig(setup->first, setup->second);
+    auto config = setupToConfig(setup->setup, setup->globalOutputIndices);
     auto internalChangeset = config.changeSet(internalOutput);
     if (!internalChangeset->enabled.value_or(internalOutput->isEnabled())) {
         return config;
@@ -539,7 +548,7 @@ std::optional<OutputConfiguration> OutputConfigurationStore::generateLidClosedCo
             }
             const auto changeset = config.changeSet(output);
             const QPoint pos = changeset->pos.value_or(output->position());
-            return QRect(pos, otherSize).intersects(QRect(otherPos, getSize(changeset.get(), output)));
+            return Rect(pos, otherSize).intersects(Rect(otherPos, getSize(changeset.get(), output)));
         });
         if (!overlap) {
             changeset->pos = otherPos;
@@ -548,6 +557,47 @@ std::optional<OutputConfiguration> OutputConfigurationStore::generateLidClosedCo
     return config;
 }
 
+std::optional<OutputConfigurationStore::SetupWithOutputs> OutputConfigurationStore::findPartialSetup(const QList<BackendOutput *> &outputs, bool lidClosed)
+{
+    std::unordered_map<BackendOutput *, size_t> outputStates;
+    for (BackendOutput *output : outputs) {
+        if (auto index = findOutputIndex(output, outputs)) {
+            outputStates[output] = *index;
+        }
+    }
+    const auto matchCount = [lidClosed, &outputStates](const Setup &setup) -> size_t {
+        if (setup.lidClosed != lidClosed) {
+            return 0;
+        }
+        // Skip the setup if it contains outputs that aren't currently connected
+        const bool otherOutputs = std::ranges::any_of(setup.outputs, [&](const SetupState &state) {
+            return std::ranges::none_of(outputStates, [&state](const auto &outputIt) {
+                return outputIt.second == state.outputIndex;
+            });
+        });
+        if (otherOutputs) {
+            return 0;
+        }
+        // now just count how many outputs are relevant
+        return std::ranges::count_if(outputStates, [&setup](const auto &outputIt) {
+            return std::ranges::any_of(setup.outputs, [&outputIt](const auto &outputInfo) {
+                return outputInfo.outputIndex == outputIt.second;
+            });
+        });
+    };
+    const auto bestSetup = std::ranges::max_element(m_setups, [&](const auto &left, const auto &right) {
+        return matchCount(left) < matchCount(right);
+    });
+    if (bestSetup == m_setups.end() || matchCount(*bestSetup) == 0) {
+        return std::nullopt;
+    } else {
+        return SetupWithOutputs{
+            .setup = &*bestSetup,
+            .globalOutputIndices = outputStates,
+        };
+    }
+}
+
 OutputConfiguration OutputConfigurationStore::generateConfig(const QList<BackendOutput *> &outputs, bool isLidClosed)
 {
     qCDebug(KWIN_OUTPUT_CONFIG, "Generating new config for %lld outputs", outputs.size());
@@ -556,15 +606,38 @@ OutputConfiguration OutputConfigurationStore::generateConfig(const QList<Backend
             return *closedConfig;
         }
     }
+    const auto closestSetup = findPartialSetup(outputs, isLidClosed);
     const auto kscreenConfig = KScreenIntegration::readOutputConfig(outputs, KScreenIntegration::connectedOutputsHash(outputs, isLidClosed));
     OutputConfiguration ret;
-    QPoint pos(0, 0);
+    QPoint rightMostPosition(0, 0);
     int priority = 0;
-    for (const auto output : outputs) {
+
+    QList<BackendOutput *> sortedOutputs = outputs;
+    if (closestSetup.has_value()) {
+        // place outputs we already have settings for first
+        std::ranges::partition(sortedOutputs, [&closestSetup](BackendOutput *output) {
+            return closestSetup->globalOutputIndices.contains(output);
+        });
+    }
+    for (BackendOutput *output : sortedOutputs) {
         const auto kscreenChangeSetPtr = kscreenConfig ? kscreenConfig->constChangeSet(output) : nullptr;
         const auto kscreenChangeSet = kscreenChangeSetPtr ? *kscreenChangeSetPtr : OutputChangeSet{};
 
-        const auto outputIndex = findOutput(output, outputs);
+        std::optional<SetupState> setupState;
+        if (closestSetup.has_value()) {
+            const auto indexIt = closestSetup->globalOutputIndices.find(output);
+            if (indexIt != closestSetup->globalOutputIndices.end()) {
+                const auto &[output, index] = *indexIt;
+                for (const SetupState &state : closestSetup->setup->outputs) {
+                    if (state.outputIndex == index) {
+                        setupState = state;
+                        break;
+                    }
+                }
+            }
+        }
+
+        const auto outputIndex = findOutputIndex(output, outputs);
         const bool enable = kscreenChangeSet.enabled.value_or(!isLidClosed || !output->isInternal() || outputs.size() == 1);
         const OutputState existingData = outputIndex ? m_outputs[*outputIndex] : OutputState{};
 
@@ -581,8 +654,8 @@ OutputConfiguration OutputConfigurationStore::generateConfig(const QList<Backend
             .mode = mode,
             .desiredModeSize = mode->size(),
             .desiredModeRefreshRate = mode->refreshRate(),
-            .enabled = kscreenChangeSet.enabled.value_or(enable),
-            .pos = pos,
+            .enabled = setupState ? setupState->enabled : enable,
+            .pos = setupState ? setupState->position : rightMostPosition,
             // kscreen scale is unreliable because it gets overwritten with the value 1 on Xorg,
             // and we don't know if it's from Xorg or the 5.27 Wayland session... so just ignore it
             .scaleSetting = existingData.scaleSetting.value_or(chooseScale(output, mode.get())),
@@ -605,12 +678,22 @@ OutputConfiguration OutputConfigurationStore::generateConfig(const QList<Backend
             .maxBitsPerColor = existingData.maxBitsPerColor,
             .edrPolicy = existingData.edrPolicy.value_or(BackendOutput::EdrPolicy::Always),
             .sharpness = existingData.sharpness.value_or(0),
-            .priority = priority,
+            .priority = setupState ? setupState->priority : priority,
+            .automaticBrightness = existingData.automaticBrightness.value_or(false),
+            // TODO generate a more fitting brightness map per screen?
+            .autoBrightnessCurve = existingData.autoBrightnessCurve,
         };
-        priority++;
-        if (enable) {
+        if (setupState) {
+            priority = std::max(setupState->priority + 1, priority);
+        } else {
+            priority++;
+        }
+        if (*changeset->enabled) {
             const auto modeSize = changeset->transform->map(mode->size());
-            pos.setX(std::ceil(pos.x() + modeSize.width() / *changeset->scaleSetting));
+            const QPoint topRight = QPoint(std::ceil(changeset->pos->x() + modeSize.width() / *changeset->scaleSetting), changeset->pos->y());
+            if (topRight.x() > rightMostPosition.x() || (topRight.x() == rightMostPosition.x() && topRight.y() < rightMostPosition.y())) {
+                rightMostPosition = topRight;
+            }
         }
     }
     return ret;
@@ -639,22 +722,26 @@ std::shared_ptr<OutputMode> OutputConfigurationStore::chooseMode(BackendOutput *
         return *std::ranges::max_element(modes, findBiggestFastest);
     }
 
-    // 32:9 displays often advertise a lower resolution mode as preferred, special case them
-    auto only32by9 = notPotentiallyBroken | std::ranges::views::filter([](const auto &mode) {
-        const double aspectRatio = mode->size().width() / double(mode->size().height());
-        return aspectRatio > 31 / 9.0 && aspectRatio < 33 / 9.0;
-    });
-    const auto best32By9 = std::ranges::max_element(only32by9, findBiggestFastest);
-    if (best32By9 != only32by9.end()) {
-        return *best32By9;
-    }
-
     // try to figure out the native resolution; the biggest preferred mode usually has that
     auto preferredOnly = notPotentiallyBroken | std::ranges::views::filter([](const auto &mode) {
         return (mode->flags() & OutputMode::Flag::Preferred);
     });
     const auto nativeSize = std::ranges::max_element(preferredOnly, findBiggestFastest);
 
+    auto is32by9 = [](const auto &mode) {
+        const double aspectRatio = mode->size().width() / double(mode->size().height());
+        return aspectRatio > 31 / 9.0 && aspectRatio < 33 / 9.0;
+    };
+
+    if (nativeSize == preferredOnly.end() || is32by9(*nativeSize)) {
+        // 32:9 displays often advertise a lower resolution mode as preferred, special case them
+        auto only32by9 = notPotentiallyBroken | std::ranges::views::filter(is32by9);
+        const auto best32By9 = std::ranges::max_element(only32by9, findBiggestFastest);
+        if (best32By9 != only32by9.end()) {
+            return *best32By9;
+        }
+    }
+
     // Non-default modes have a decent chance of not working on VGA,
     // so avoid doing anything out of the ordinary there
     const bool isVGA = output->name().contains("VGA");
@@ -761,7 +848,7 @@ void OutputConfigurationStore::registerOutputs(const QList<BackendOutput *> &out
         if (output->isNonDesktop() || output->isPlaceholder()) {
             continue;
         }
-        auto index = findOutput(output, outputs);
+        auto index = findOutputIndex(output, outputs);
         if (!index) {
             index = m_outputs.size();
             m_outputs.push_back(OutputState{});
@@ -1042,6 +1129,12 @@ void OutputConfigurationStore::load()
             }
             state.customModes = modes;
         }
+        if (const auto it = data.find("automaticBrightness"); it != data.end() && it->isBool()) {
+            state.automaticBrightness = it->toBool();
+        }
+        if (const auto it = data.find("autoBrightnessCurve"); it != data.end() && it->isArray()) {
+            state.autoBrightnessCurve = AutoBrightnessCurve::fromArray(it->toArray());
+        }
         outputDatas.push_back(state);
     }
 
@@ -1333,12 +1426,18 @@ void OutputConfigurationStore::save()
             for (const auto &mode : *output.customModes) {
                 QJsonObject obj;
                 obj["width"] = mode.size.width();
-                obj["height"] = mode.size.width();
+                obj["height"] = mode.size.height();
                 obj["refreshRate"] = int(mode.refreshRate);
                 obj["flags"] = int(mode.flags);
                 modes.append(obj);
             }
         }
+        if (output.automaticBrightness) {
+            o["automaticBrightness"] = *output.automaticBrightness;
+        }
+        if (output.autoBrightnessCurve) {
+            o["autoBrightnessCurve"] = output.autoBrightnessCurve->toArray();
+        }
         outputsData.append(o);
     }
     outputs["data"] = outputsData;
@@ -1406,4 +1505,13 @@ bool OutputConfigurationStore::isAutoRotateActive(const QList<BackendOutput *> &
     }
     Q_UNREACHABLE();
 }
+
+bool OutputConfigurationStore::isAutoBrightnessActive(const QList<BackendOutput *> &outputs) const
+{
+    return std::ranges::any_of(outputs, [](BackendOutput *output) {
+        return output->isEnabled()
+            && output->automaticBrightness()
+            && output->dpmsMode() == BackendOutput::DpmsMode::On;
+    });
+}
 }
