@@ -4,6 +4,7 @@
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 #include "colormanagement_v1.h"
+#include "clientconnection.h"
 #include "display.h"
 #include "surface.h"
 #include "surface_p.h"
@@ -15,6 +16,7 @@ namespace KWin
 
 ColorManagerV1::ColorManagerV1(Display *display, QObject *parent)
     : QObject(parent)
+    // HACK to work around https://bugzilla.mozilla.org/show_bug.cgi?id=2008777 in Firefox based software (Thunderbird and Zen Browser)
     , QtWaylandServer::wp_color_manager_v1(*display, 1)
 {
 }
@@ -43,12 +45,17 @@ void ColorManagerV1::wp_color_manager_v1_bind_resource(Resource *resource)
     send_supported_tf_named(resource->handle, transfer_function::transfer_function_st2084_pq);
     send_supported_tf_named(resource->handle, transfer_function::transfer_function_ext_linear);
     send_supported_tf_named(resource->handle, transfer_function::transfer_function_bt1886);
+    if (resource->version() >= WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_COMPOUND_POWER_2_4_SINCE_VERSION) {
+        send_supported_tf_named(resource->handle, transfer_function::transfer_function_compound_power_2_4);
+    }
 
     send_supported_intent(resource->handle, render_intent::render_intent_perceptual);
     send_supported_intent(resource->handle, render_intent::render_intent_relative);
     send_supported_intent(resource->handle, render_intent::render_intent_absolute);
     send_supported_intent(resource->handle, render_intent::render_intent_relative_bpc);
-    // TODO implement saturation intent
+    if (resource->version() >= WP_COLOR_MANAGER_V1_RENDER_INTENT_ABSOLUTE_NO_ADAPTATION_SINCE_VERSION) {
+        send_supported_intent(resource->handle, render_intent::render_intent_absolute_no_adaptation);
+    }
 
     send_done(resource->handle);
 }
@@ -103,7 +110,7 @@ void ColorManagerV1::wp_color_manager_v1_create_windows_scrgb(Resource *resource
         Colorimetry::BT2020,
         Colorimetry::BT709,
     });
-    ImageDescriptionV1::createReady(resource->client(), image_description, resource->version(), scrgb);
+    ImageDescriptionV1::createReady(resource->client(), image_description, resource->version(), scrgb, ColorDescriptionType::Windows);
 }
 
 ColorFeedbackSurfaceV1::ColorFeedbackSurfaceV1(wl_client *client, uint32_t id, uint32_t version, SurfaceInterface *surface)
@@ -120,11 +127,26 @@ ColorFeedbackSurfaceV1::~ColorFeedbackSurfaceV1()
     }
 }
 
+static uint32_t highPart(uint64_t number)
+{
+    return number >> 32;
+}
+
+static uint32_t lowPart(uint64_t number)
+{
+    return number & 0xFFFFFFFF;
+}
+
 void ColorFeedbackSurfaceV1::setPreferredColorDescription(const std::shared_ptr<ColorDescription> &descr)
 {
     if (m_preferred != descr) {
         m_preferred = descr;
-        send_preferred_changed(resource()->handle, ImageDescriptionV1::s_idCounter++);
+        ImageDescriptionV1::s_idCounter++;
+        if (resource()->version() >= WP_COLOR_MANAGEMENT_SURFACE_FEEDBACK_V1_PREFERRED_CHANGED2_SINCE_VERSION) {
+            send_preferred_changed2(resource()->handle, highPart(ImageDescriptionV1::s_idCounter), lowPart(ImageDescriptionV1::s_idCounter));
+        } else {
+            send_preferred_changed(resource()->handle, lowPart(ImageDescriptionV1::s_idCounter));
+        }
     }
 }
 
@@ -140,12 +162,12 @@ void ColorFeedbackSurfaceV1::wp_color_management_surface_feedback_v1_destroy(Res
 
 void ColorFeedbackSurfaceV1::wp_color_management_surface_feedback_v1_get_preferred(Resource *resource, uint32_t image_description)
 {
-    ImageDescriptionV1::createReady(resource->client(), image_description, resource->version(), m_preferred);
+    ImageDescriptionV1::createReady(resource->client(), image_description, resource->version(), m_preferred, ColorDescriptionType::Normal);
 }
 
 void ColorFeedbackSurfaceV1::wp_color_management_surface_feedback_v1_get_preferred_parametric(Resource *resource, uint32_t image_description)
 {
-    ImageDescriptionV1::createReady(resource->client(), image_description, resource->version(), m_preferred);
+    ImageDescriptionV1::createReady(resource->client(), image_description, resource->version(), m_preferred, ColorDescriptionType::Normal);
 }
 
 ColorSurfaceV1::ColorSurfaceV1(wl_client *client, uint32_t id, uint32_t version, SurfaceInterface *surface)
@@ -159,6 +181,7 @@ ColorSurfaceV1::~ColorSurfaceV1()
     if (m_surface) {
         const auto priv = SurfaceInterfacePrivate::get(m_surface);
         priv->pending->colorDescription = ColorDescription::sRGB;
+        priv->pending->colorDescriptionType = ColorDescriptionType::Normal;
         priv->pending->committed |= SurfaceState::Field::ColorDescription;
         priv->colorSurface = nullptr;
     }
@@ -182,6 +205,8 @@ static std::optional<RenderingIntent> waylandToKwinIntent(uint32_t intent)
     case QtWaylandServer::wp_color_manager_v1::render_intent::render_intent_relative:
     case QtWaylandServer::wp_color_manager_v1::render_intent::render_intent_absolute:
         return RenderingIntent::RelativeColorimetric;
+    case QtWaylandServer::wp_color_manager_v1::render_intent::render_intent_absolute_no_adaptation:
+        return RenderingIntent::AbsoluteColorimetricNoAdaptation;
     case QtWaylandServer::wp_color_manager_v1::render_intent::render_intent_relative_bpc:
         return RenderingIntent::RelativeColorimetricWithBPC;
     case QtWaylandServer::wp_color_manager_v1::render_intent::render_intent_saturation:
@@ -207,8 +232,9 @@ void ColorSurfaceV1::wp_color_management_surface_v1_set_image_description(Resour
         return;
     }
     const auto priv = SurfaceInterfacePrivate::get(m_surface);
-    priv->pending->colorDescription = *ImageDescriptionV1::get(image_description)->description();
+    priv->pending->colorDescription = *description->description();
     priv->pending->renderingIntent = *intent;
+    priv->pending->colorDescriptionType = description->type();
     priv->pending->committed |= SurfaceState::Field::ColorDescription;
 }
 
@@ -220,6 +246,7 @@ void ColorSurfaceV1::wp_color_management_surface_v1_unset_image_description(Reso
     }
     const auto priv = SurfaceInterfacePrivate::get(m_surface);
     priv->pending->colorDescription = ColorDescription::sRGB;
+    priv->pending->colorDescriptionType = ColorDescriptionType::Normal;
     priv->pending->committed |= SurfaceState::Field::ColorDescription;
 }
 
@@ -265,7 +292,15 @@ void ColorParametricCreatorV1::wp_image_description_creator_params_v1_create(Res
         maxHdrLuminance = 1'000;
         m_minMasteringLuminance = func.minLuminance;
     }
+
     if (Colorimetry::isValid(m_colorimetry->red().toxy(), m_colorimetry->green().toxy(), m_colorimetry->blue().toxy(), m_colorimetry->white().toxy())) {
+        auto type = ColorDescriptionType::Normal;
+        ClientConnection *client = ClientConnection::get(resource->client());
+        // TODO remove gamescope from that list, once we have a Windows_PQ image description it can use
+        if (client->executablePath().endsWith("wine-preloader") || client->executablePath().endsWith("gamescope")) {
+            type = ColorDescriptionType::Windows;
+        }
+
         ImageDescriptionV1::createReady(resource->client(), image_description, resource->version(), std::make_shared<ColorDescription>(ColorDescription{
                                                                                                         *m_colorimetry,
                                                                                                         func,
@@ -275,7 +310,8 @@ void ColorParametricCreatorV1::wp_image_description_creator_params_v1_create(Res
                                                                                                         maxHdrLuminance.value_or(func.maxLuminance),
                                                                                                         m_masteringColorimetry.value_or(*m_colorimetry),
                                                                                                         Colorimetry::BT709,
-                                                                                                    }));
+                                                                                                    }),
+                                        type);
     } else {
         ImageDescriptionV1::createFailed(resource->client(), image_description, resource->version(), WP_IMAGE_DESCRIPTION_V1_CAUSE_UNSUPPORTED, QStringLiteral("The provided image description failed to verify as usable"));
     }
@@ -301,6 +337,9 @@ void ColorParametricCreatorV1::wp_image_description_creator_params_v1_set_tf_nam
     case WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_BT1886:
         m_transferFunctionType = TransferFunction::BT1886;
         return;
+    case WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_COMPOUND_POWER_2_4:
+        m_transferFunctionType = TransferFunction::sRGB;
+        return;
         // intentionally not supported - it's confusing and
         // deprecated in version 2 of the protocol
     case WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_SRGB:
@@ -446,23 +485,29 @@ void ColorParametricCreatorV1::wp_image_description_creator_params_v1_set_max_fa
 
 uint64_t ImageDescriptionV1::s_idCounter = 1;
 
-ImageDescriptionV1 *ImageDescriptionV1::createReady(wl_client *client, uint32_t id, uint32_t version, const std::shared_ptr<ColorDescription> &colorDescription)
+ImageDescriptionV1 *ImageDescriptionV1::createReady(wl_client *client, uint32_t id, uint32_t version, const std::shared_ptr<ColorDescription> &colorDescription, ColorDescriptionType type)
 {
-    auto description = new ImageDescriptionV1(client, id, version, colorDescription);
-    description->send_ready(s_idCounter++);
+    auto description = new ImageDescriptionV1(client, id, version, colorDescription, type);
+    s_idCounter++;
+    if (version >= WP_IMAGE_DESCRIPTION_V1_READY2_SINCE_VERSION) {
+        description->send_ready2(highPart(s_idCounter), lowPart(s_idCounter));
+    } else {
+        description->send_ready(lowPart(s_idCounter));
+    }
     return description;
 }
 
 ImageDescriptionV1 *ImageDescriptionV1::createFailed(wl_client *client, uint32_t id, uint32_t version, wp_image_description_v1_cause error, const QString &message)
 {
-    auto description = new ImageDescriptionV1(client, id, version, std::nullopt);
+    auto description = new ImageDescriptionV1(client, id, version, std::nullopt, ColorDescriptionType::Normal);
     description->send_failed(error, message);
     return description;
 }
 
-ImageDescriptionV1::ImageDescriptionV1(wl_client *client, uint32_t id, uint32_t version, const std::optional<std::shared_ptr<ColorDescription>> &color)
+ImageDescriptionV1::ImageDescriptionV1(wl_client *client, uint32_t id, uint32_t version, const std::optional<std::shared_ptr<ColorDescription>> &color, ColorDescriptionType type)
     : QtWaylandServer::wp_image_description_v1(client, id, version)
     , m_description(color)
+    , m_type(type)
 {
 }
 
@@ -483,8 +528,8 @@ static uint32_t kwinTFtoProtoTF(TransferFunction tf)
         return WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_EXT_LINEAR;
     case TransferFunction::PerceptualQuantizer:
         return WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_ST2084_PQ;
-        // not "correct", but the piece-wise TF shouldn't be used anyways
     case TransferFunction::sRGB:
+        return WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_COMPOUND_POWER_2_4;
     case TransferFunction::gamma22:
         return WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_GAMMA22;
     case TransferFunction::BT1886:
@@ -590,6 +635,11 @@ const std::optional<std::shared_ptr<ColorDescription>> &ImageDescriptionV1::desc
     return m_description;
 }
 
+ColorDescriptionType ImageDescriptionV1::type() const
+{
+    return m_type;
+}
+
 ImageDescriptionV1 *ImageDescriptionV1::get(wl_resource *resource)
 {
     if (auto resourceContainer = Resource::fromResource(resource)) {
@@ -625,7 +675,7 @@ void ColorManagementOutputV1::wp_color_management_output_v1_get_image_descriptio
     if (!m_output || m_output->isRemoved()) {
         ImageDescriptionV1::createFailed(resource->client(), image_description, resource->version(), WP_IMAGE_DESCRIPTION_V1_CAUSE_NO_OUTPUT, QStringLiteral("wl_output has been removed"));
     } else {
-        ImageDescriptionV1::createReady(resource->client(), image_description, resource->version(), m_output->handle()->blendingColor());
+        ImageDescriptionV1::createReady(resource->client(), image_description, resource->version(), m_output->handle()->blendingColor(), ColorDescriptionType::Normal);
     }
 }
 
