@@ -25,7 +25,7 @@
 namespace KWin
 {
 
-static const quint32 s_version = 18;
+static const quint32 s_version = 19;
 
 class OutputManagementV2InterfacePrivate : public QtWaylandServer::kde_output_management_v2
 {
@@ -79,6 +79,7 @@ protected:
     void kde_output_configuration_v2_set_edr_policy(Resource *resource, struct ::wl_resource *outputdevice, uint32_t edrPolicy) override;
     void kde_output_configuration_v2_set_sharpness(Resource *resource, wl_resource *outputdevice, uint32_t sharpness) override;
     void kde_output_configuration_v2_set_custom_modes(Resource *resource, struct ::wl_resource *outputdevice, struct ::wl_resource *modes) override;
+    void kde_output_configuration_v2_set_auto_brightness(Resource *resource, ::wl_resource *outputdevice, uint32_t enabled) override;
 
     void sendFailure(Resource *resource, const QString &reason);
 };
@@ -408,6 +409,7 @@ void OutputConfigurationV2Interface::kde_output_configuration_v2_set_brightness(
     }
     if (OutputDeviceV2Interface *output = OutputDeviceV2Interface::get(outputdevice)) {
         config.changeSet(output->handle())->brightness = brightness / 10'000.0;
+        config.changeSet(output->handle())->brightnessReason = BackendOutput::BrightnessReason::ManualAdjustment;
     }
 }
 
@@ -517,6 +519,16 @@ void OutputConfigurationV2Interface::kde_output_configuration_v2_set_custom_mode
     config.changeSet(output->handle())->customModes = r->modes;
 }
 
+void OutputConfigurationV2Interface::kde_output_configuration_v2_set_auto_brightness(Resource *resource, ::wl_resource *outputdevice, uint32_t enabled)
+{
+    if (invalid) {
+        return;
+    }
+    if (OutputDeviceV2Interface *output = OutputDeviceV2Interface::get(outputdevice)) {
+        config.changeSet(output->handle())->automaticBrightness = bool(enabled);
+    }
+}
+
 void OutputConfigurationV2Interface::kde_output_configuration_v2_destroy(Resource *resource)
 {
     wl_resource_destroy(resource->handle);
@@ -626,8 +638,15 @@ void OutputConfigurationV2Interface::kde_output_configuration_v2_apply(Resource
             }
         }
         if (changeset->pos.has_value()) {
-            if (changeset->pos->x() < 0 || changeset->pos->y() < 0) {
-                sendFailure(resource, i18n("Position of output %s is negative, that is not supported", output->name()));
+            // KScreen in some cases moves disabled screens to negative positions, to preserve their
+            // relative position vs. the still enabled outputs. Until that's changed, we have to allow
+            // disabled outputs to be in negative positions
+            const bool enabled = changeset->enabled.value_or(output->isEnabled());
+            if (!enabled && (changeset->pos->x() < -1000000 || changeset->pos->y() < -1000000)) {
+                sendFailure(resource, i18n("Position of output %s is way too negative (%d, %d)", output->name(), changeset->pos->x(), changeset->pos->y()));
+                return;
+            } else if (enabled && (changeset->pos->x() < 0 || changeset->pos->y() < 0)) {
+                sendFailure(resource, i18n("Position of enabled output %s is negative (%d, %d)", output->name(), changeset->pos->x(), changeset->pos->y()));
                 return;
             }
             if (changeset->pos->x() > 1000000 || changeset->pos->y() > 1000000) {
@@ -637,6 +656,7 @@ void OutputConfigurationV2Interface::kde_output_configuration_v2_apply(Resource
         }
     }
 
+    config.source = OutputConfiguration::Source::User;
     switch (workspace()->applyOutputConfiguration(config)) {
     case OutputConfigurationError::None:
         send_applied();
