@@ -200,7 +200,7 @@ void SeatInterface::endDrag()
     AbstractDataSource *dragSource = d->drag.source;
 
     if (dragSource) {
-        if (dragTargetDevice && dragSource->isAccepted() && dragSource->selectedDndAction() != DataDeviceManagerInterface::DnDAction::None) {
+        if (dragTargetDevice && dragSource->isAccepted() && dragSource->selectedDndAction() != DnDAction::None) {
             Q_EMIT dragDropped();
             dragTargetDevice->drop();
             dragSource->dropPerformed();
@@ -250,7 +250,7 @@ void SeatInterfacePrivate::registerDataControlDevice(DataControlDeviceV1Interfac
         // If the mimetype x-kde-onlyReplaceEmpty is set, and we've had another update in the meantime, do nothing
         // but resend selection to mimic normal event flow upon cancel and not confuse the client
         // See https://github.com/swaywm/wlr-protocols/issues/92
-        const bool isKlipperEmptyReplacement = dataDevice->selection() && dataDevice->selection()->mimeTypes().contains(QLatin1String("application/x-kde-onlyReplaceEmpty"));
+        const bool isKlipperEmptyReplacement = dataDevice->selection() && dataDevice->selection()->mimeTypes().contains(QLatin1StringView("application/x-kde-onlyReplaceEmpty"));
         if (isKlipperEmptyReplacement && currentSelection && !currentSelection->mimeTypes().isEmpty()) {
             dataDevice->selection()->cancel();
             offerSelection(dataDevice);
@@ -264,7 +264,7 @@ void SeatInterfacePrivate::registerDataControlDevice(DataControlDeviceV1Interfac
         // If the mimetype x-kde-onlyReplaceEmpty is set, and we've had another update in the meantime, do nothing
         // but resend selection to mimic normal event flow upon cancel and not confuse the client
         // See https://github.com/swaywm/wlr-protocols/issues/92
-        const bool isKlipperEmptyReplacement = dataDevice->primarySelection() && dataDevice->primarySelection()->mimeTypes().contains(QLatin1String("application/x-kde-onlyReplaceEmpty"));
+        const bool isKlipperEmptyReplacement = dataDevice->primarySelection() && dataDevice->primarySelection()->mimeTypes().contains(QLatin1StringView("application/x-kde-onlyReplaceEmpty"));
         if (isKlipperEmptyReplacement && currentPrimarySelection && !currentPrimarySelection->mimeTypes().isEmpty()) {
             dataDevice->primarySelection()->cancel();
             offerPrimarySelection(dataDevice);
@@ -308,9 +308,9 @@ bool SeatInterfacePrivate::dragInhibitsPointer(SurfaceInterface *surface) const
     return targetHasDataDevice;
 }
 
-void SeatInterfacePrivate::updateSelection(DataSourceInterface *dataSource, quint32 serial)
+void SeatInterfacePrivate::updateSelection(DataSourceInterface *dataSource, UInt32Serial serial)
 {
-    if (currentSelectionSerial - serial < UINT32_MAX / 2 && currentSelectionSerial != serial) {
+    if (serial < currentSelectionSerial) {
         if (dataSource) {
             dataSource->cancel();
         }
@@ -319,9 +319,9 @@ void SeatInterfacePrivate::updateSelection(DataSourceInterface *dataSource, quin
     q->setSelection(dataSource, serial);
 }
 
-void SeatInterfacePrivate::updatePrimarySelection(PrimarySelectionSourceV1Interface *dataSource, quint32 serial)
+void SeatInterfacePrivate::updatePrimarySelection(PrimarySelectionSourceV1Interface *dataSource, UInt32Serial serial)
 {
-    if (currentPrimarySelectionSerial - serial < UINT32_MAX / 2 && currentPrimarySelectionSerial != serial) {
+    if (serial < currentPrimarySelectionSerial) {
         if (dataSource) {
             dataSource->cancel();
         }
@@ -1156,7 +1156,7 @@ AbstractDataSource *SeatInterface::selection() const
     return d->currentSelection;
 }
 
-void SeatInterface::setSelection(AbstractDataSource *selection, quint32 serial)
+void SeatInterface::setSelection(AbstractDataSource *selection, UInt32Serial serial)
 {
     if (d->currentSelection == selection) {
         return;
@@ -1196,7 +1196,7 @@ AbstractDataSource *SeatInterface::primarySelection() const
     return d->currentPrimarySelection;
 }
 
-void SeatInterface::setPrimarySelection(AbstractDataSource *selection, quint32 serial)
+void SeatInterface::setPrimarySelection(AbstractDataSource *selection, UInt32Serial serial)
 {
     if (d->currentPrimarySelection == selection) {
         return;
